/************************************************************************
 *
 *  (C) Copyright 2001-2006 Novell, Inc.
 *  All Rights Reserved.
 *
 *  This program is an unpublished copyrighted work which is proprietary
 *  to Novell, Inc. and contains confidential information that is not
 *  to be reproduced or disclosed to any other person or entity without
 *  prior written consent from Novell, Inc. in each and every instance.
 *
 *  WARNING:  Unauthorized reproduction of this program as well as
 *  unauthorized preparation of derivative works based upon the
 *  program or distribution of copies by sale, rental, lease or
 *  lending are violations of federal copyright laws and state trade
 *  secret laws, punishable by civil and criminal penalties.
 *
 ****************************************************************************/
#ifndef _SASL_H
#define _SASL_H

#include <nmassasl.h>
#include <saslapi.h>

#define SASL_MECH_NMAS_LOGIN        "NMAS_LOGIN"

#define SASLMechanismNames NMASMechanismNames

SASLAPI SASLMechanismNames * SASL_GetMechanisms(void);

#define SASL_LOCAL_CONTEXT      0
#define SASL_FULL_CONTEXT       1
#define SASL_NO_CONTEXT         2

SASLAPI int SASL_DoMechanism(
    char *mechanism,        // mechanism name
    char *dn,               // full ldap object DN (optional)
    int requestedCxtType,   // one of the context types defined above
    int dContext,       // DClient context which will get login identity
    int *actualCxtType,     // one of the context types defined above
    nuint32 inBufferSize,   // size of mechanism specific data received from client
    char *inBuffer,         // mechanism specific input received from client
    nuint32 *outBufferSize, // size of mechanism specific data to be sent to client
    char **outBuffer,       // mechanism specific data to be sent to client
    nuint32 *saslID,         // SASL session ID
    nuint32 *status);       // status of SASL Session

SASLAPI void SASL_FreeReply(char *buffer);

SASLAPI int SASL_UpgradeContext(
    nuint32 saslID,      // SASL session ID
    int dContext);      // DClient context

SASLAPI void SASL_CloseSession(
    nuint32 saslID);     // SASL ID for session to be closed

// Returns the Security Layer hande that is needed of the SASL_DoDecode and SASL_DoEncode
SASLAPI int SASL_GetSecLayerHandle(
    nuint32 saslID,          // SASL session ID
    void **slHandle);       // Security Layer session handle

SASLAPI int SASL_DoDecode(
    void *slHandle,         // Security Layer session handle
    nuint32 inFlags,        // options flags -- currently must be zero
    nuint32 inBufferSize,   // the size in bytes of the encoded input buffer
    char *inBuffer,         // the encoded input buffer
    nuint32 *outFlags,      // status flags -- currently NMAS_SEC_LAYER_STATUS_INTEG and/or NMAS_SEC_LAYER_STATUS_CONF
    nuint32 *outBufferSize, // in: size in bytes of the output buffer -- out: number of bytes written to output buffer
    char **outBuffer);      // the output buffer

SASLAPI int SASL_DoEncode(
    void *slHandle,         // Security Layer session handle
    nuint32 inFlags,        // options flags -- currently must be zero
    nuint32 inBufferSize,   // the size in bytes of the input buffer
    char *inBuffer,         // the input buffer
    nuint32 *outFlags,      // status flags -- currently NMAS_SEC_LAYER_STATUS_INTEG and/or NMAS_SEC_LAYER_STATUS_CONF
    nuint32 *outBufferSize, // in: size in bytes of the output buffer -- out: number of bytes written to output buffer
    char **outBuffer);      // the output buffer

#endif

