/****************************************************************************

  %name:          dislogin.h %
  %version:       3 %
  %date_modified: Mon Aug  1 16:00:23 2011 %
  $Copyright: 

  Copyright (c) 2010 Novell, Inc.  All Rights Reserved.                      

  THIS WORK IS  SUBJECT  TO  U.S.  AND  INTERNATIONAL  COPYRIGHT  LAWS  AND
  TREATIES.   NO  PART  OF  THIS  WORK MAY BE  USED,  PRACTICED,  PERFORMED
  COPIED, DISTRIBUTED, REVISED, MODIFIED, TRANSLATED,  ABRIDGED, CONDENSED,
  EXPANDED,  COLLECTED,  COMPILED,  LINKED,  RECAST, TRANSFORMED OR ADAPTED
  WITHOUT THE PRIOR WRITTEN CONSENT OF NOVELL, INC. ANY USE OR EXPLOITATION
  OF THIS WORK WITHOUT AUTHORIZATION COULD SUBJECT THE PERPETRATOR TO
  CRIMINAL AND CIVIL LIABILITY.
   
*****************************************************************************/
#ifndef _DIS_LOGIN_H_
#define _DIS_LOGIN_H_

#define MAX_TREE_NAME_LEN                      255
#define MAX_DN_LEN		      				   256
#define MAX_METHOD_NAME_LEN                    11		//max int - 4 bytes

#define NMAS_CONFIGURED_METHOD_NAME				L"NMAS_CONFIGURED_METHOD_NAME"

typedef struct NMAS_edirInfo
{
	wchar_t wszEdirUName[MAX_DN_LEN + 1];
    wchar_t wszMethodName[MAX_DN_LEN + 1];
	wchar_t wszTreeName[MAX_TREE_NAME_LEN + 1];
}NMAS_edirInfo;

enum NMAS_DIS_LOGIN_ENROLLMENT
{
	NOT_ENROLLED = 0,
	ENROLLED
};

enum NMAS_DIS_LOGIN_ENFORCED
{
	NOT_ENFORCED = 0,
	ENFORCED
};


/*
* Function signature for lcm's locallogin function
*/
typedef int (*pFunc_LLMethod)(wchar_t* pwszPin, NMAS_edirInfo* pedirInfo );

/*
* Function signature for lcm's disconnected status query function
*/
typedef int (*pFunc_DisStatQuery)(DWORD* pdwEnrollment, DWORD* pdwEnforcement);


#if defined(__cplusplus)
extern "C" {
#endif

/*
* API to perform disconnected login
*/
int NMAS_RetrieveLocalLoginCredentials( wchar_t* pwszWDomainName, wchar_t* pwszWUName, 
						                wchar_t* pwszWPin, wchar_t** ppwszWPass, DWORD* pdwEnforced );

/*
* Store user credentials to PassStore for disconnected login
*/
int NMAS_PutLocalSecretForDL(wchar_t* pwszWDomainName, wchar_t* pwszWUName,
							 wchar_t* pwszPassword, wchar_t* pwszEdirUName, 
							 wchar_t* pwszTreeName);

/*
* Store user credentials to PassStore for single signon
*/
int NMAS_PutLocalSecretForSS(wchar_t* pwszWDomainName, wchar_t* pwszWUName,
							 wchar_t* pwszPassword, wchar_t* pwszEdirUName, 
							 wchar_t* pwszTreeName);

/*
* Get credentials from PassStore
* Caller to free the password buffer using free(call)
*/
int NMAS_GetLocalSecretForSS(wchar_t* pwszEdirUName, wchar_t* pwszTreeName,
							 wchar_t* pwszWDomainName, wchar_t* pwszWUName,
							 wchar_t** ppwszWPass);

/*
* Store user data in key-value format in PassStore
* input length in characters
*/
int NMAS_PutData(wchar_t* pwszKey, unsigned int nKeyLen, wchar_t* pwszValue, unsigned int nValLen);

/*
* Get user data from PassStore
* Caller to free the out buffer using free(call)
* input length in characters
*/
int NMAS_GetData(wchar_t* pwszKey, unsigned int nKeyLen, wchar_t** ppwszValue);

/*
* Get Enrollemnt status 
* Returns enrolled if at least one of the methods is enrolled for disconnected login
*/
int NMAS_GetEnrollmentStatus(DWORD* pdwEnrollment);

/*
* Free Meemory allocated by this library
* Accepts NULL as a valid argument
*/
void NMAS_FreeLocalLoginBuff(wchar_t* pwszBuff);

//Error codes
#define NMAS_E_BASE2										(-16000)
#define NMAS_E_ENFORCED_LOCAL_LOGIN_FAILED					(NMAS_E_BASE2-55) /* -16055   0xFFFFC14B */
#define NMAS_E_LOCAL_LOGIN_FAILED							(NMAS_E_BASE2-56) /* -16056   0xFFFFC14C */
#define NMAS_NO_METHOD_AVAILABLE_FOR_DIS_LOGIN				(NMAS_E_BASE2-57) /* -16057   0xFFFFC14D */

#define LOCAL_SECRET_CORRUPT 			-1
#define LOCAL_SECRET_NOMEM 				-3
#define LOCAL_SECRET_READ_FAIL			-4
#define LOCAL_SECRET_NO_EXIST			-5
#define LOCAL_SECRET_EMPTY				-6
#define LOCAL_SECRET_BLOB_OVERFLOW 		-7
#define LOCAL_SECRET_WRONG_VERSION		-8
#define LOCAL_SECRET_VALUENOFIND 		-9


#if defined(__cplusplus)
}
#endif

#endif                  //_DIS_LOGIN_H_