/******************************************************************************

  Copyright (c) 1997-2007 Novell, Inc.  All Rights Reserved.                      

  THIS WORK IS  SUBJECT  TO  U.S.  AND  INTERNATIONAL  COPYRIGHT  LAWS  AND
  TREATIES.   NO  PART  OF  THIS  WORK MAY BE  USED,  PRACTICED,  PERFORMED
  COPIED, DISTRIBUTED, REVISED, MODIFIED, TRANSLATED,  ABRIDGED, CONDENSED,
  EXPANDED,  COLLECTED,  COMPILED,  LINKED,  RECAST, TRANSFORMED OR ADAPTED
  WITHOUT THE PRIOR WRITTEN CONSENT OF NOVELL, INC. ANY USE OR EXPLOITATION
  OF THIS WORK WITHOUT AUTHORIZATION COULD SUBJECT THE PERPETRATOR TO
  CRIMINAL AND CIVIL LIABILITY.

 *****************************************************************************/
 
#ifndef	_MAF_H
#define	_MAF_H

/* WARNING:  This header does not support 64-bit development 
*
*  Windows:  _WIN64
*  Sun:      _LP64
*  Linux:    __LP64__  (GNUC)
*  HPUX:     __LP64__ 
*  AIX:      __64BIT__
*/
#if (defined(_WIN64) || defined(_LP64) || defined(__LP64__) || defined (__64BIT__))
#error Header does not support 64-bit development
#endif  
            

#include <nmasapi.h>
#include <unicode.h>

#ifndef	MAF_Handle
#define	MAF_Handle	void *
#endif

#define GLOBAL_MID_LEN	4
#define GLOBAL_MID		0x00000000

#define NMAS_AID_USERNAME						1
#define NMAS_AID_TREENAME						2
#define NMAS_AID_DS_CONTEXT						3
#define NMAS_AID_REQUESTED_READ_CLEARANCE		4  // name is misleading -- replace with following name
#define NMAS_AID_REQUESTED_CLEARANCE			4  // more meaningful name
#define NMAS_AID_REQUESTED_WRITE_CLEARANCE		5
#define NMAS_AID_PASSWORD						6
#define NMAS_AID_PIN							7
#define NMAS_AID_NAME_SPACE						8
#define NMAS_AID_ROLE							9
#define NMAS_AID_CLEARANCE_FLAG					10
#define NMAS_AID_REQUESTED_METHODS				11  // name is misleading -- replace with following name
#define NMAS_AID_REQUESTED_LOGIN_SEQUENCE		11	// more meaningful name
#define NMAS_AID_MAF_CONTEXT					12
#define NMAS_AID_SERVER_MAF_HANDLE				13
#define NMAS_AID_CLIENT_MAF_HANDLE				14
#define NMAS_AID_SESSION_CONTEXT				15
#define NMAS_AID_CONNECTION						16
#define NMAS_AID_LDAP_CONTEXT					17
#define NMAS_AID_LM_CONFIG_DATA					18
#define NMAS_AID_LM_SECRET_DATA					19
#define NMAS_AID_LD_CONFIG_DATA					20
#define NMAS_AID_LD_SECRET_DATA					21
#define NMAS_AID_USER_CONFIG_DATA				22
#define NMAS_AID_USER_SECRET_DATA				23
#define NMAS_AID_USER_GLOBAL_CONFIG_DATA		24
#define NMAS_AID_USER_GLOBAL_SECRET_DATA		25
#define NMAS_AID_NETWORK_ADDRESS				26
#define NMAS_AID_LCM_CALLBACK					27
#define NMAS_AID_GA_INFO						28
#define NMAS_AID_DISCONNECTED_FLAG				29
#define NMAS_AID_LM_DN							30
#define NMAS_AID_LM_GLOBAL_CONFIG_DATA			31
#define NMAS_AID_LM_GLOBAL_SECRET_DATA			32
#define NMAS_AID_GET_PASSWORD_POLICY_DN			33
#define NMAS_AID_PROMPT_FOR_PASSWORD			34
#define NMAS_AID_OPTIONS                     35
#define NMAS_AID_FEATURES                    36
#define NMAS_AID_PWD_WARNING                 37
#define NMAS_AID_VERIFY_ONLY                 38

// MAF_LAST_PREDEFINED_ATTRIBUTE must be the value of the last predefined MAF attribute ID above
#define MAF_LAST_PREDEFINED_ATTRIBUTE NMAS_AID_VERIFY_ONLY

/*	Prototypes of MAF Module support routines	*/

NMASAPI	nint32  MAF_Begin(MAF_Handle mh);
NMASAPI	nint32  MAF_Read(MAF_Handle mh, nint32 *rLenp, void *rBufp);
NMASAPI	nint32  MAF_WriteRead(MAF_Handle mh, nint32 wLen, void *wBufp, nint32 *wLenp, void *rBufp);
NMASAPI	nint32  MAF_Write(MAF_Handle nh, nint32 wLen, void *wBufp);
NMASAPI	nint32  MAF_End(MAF_Handle mh, nint32 status, nint32 len, void *seedDatap);

NMASAPI	nint32  MAF_XRead(MAF_Handle mh, nint32 *rLenp, void *rBufp);
NMASAPI	nint32  MAF_XWriteRead(MAF_Handle mh, nint32 wLen, void *wBufp, nint32 *wLenp, void *rBufp);
NMASAPI	nint32  MAF_XWrite(MAF_Handle nh, nint32 wLen, void *wBufp);

NMASAPI	void 	*MAF_Malloc(size_t s);
NMASAPI	void 	*MAF_Realloc(void *p, size_t s);
NMASAPI	void	MAF_Free(void *p);

NMASAPI	nint32	MAF_GetAttribute(MAF_Handle mh, nint32 aid, void *uniTag, nint32 *aLenInBytesp, void *aValuep);
NMASAPI	nint32	MAF_PutAttribute(MAF_Handle mh, nint32 aid, void *uniTag, nint32 aLenInBytes, void *aValuep);

NMASAPI	nint32	MAF_RecallLCM(MAF_Handle mh, nuint32 methodNumber, nuint32 methodData);

NMASAPI nint32	MAF_SetIdentity(MAF_Handle mh, nint reserved, nint ndsContext);

// Flags that may be passed to the MAF set password function from a SPM agent -- also see spm.h
#define SPM_ADMIN_REQUEST 	0x1			// Indicates that the password change request is not from the user
#define SPM_IGNORE_POLICY	0x8			// Indicates that the password policy will not be checked on the input password
#define SPM_EXPIRE_NOW		0x10		// Indicates that the new password will be marked expired

// Flags that may be returned from the SPM get password function -- also see spm.h
#define SPM_EXPIRED				0x1
#define SPM_GRACE_LOGIN_LIMIT	0x2
#define SPM_SPWD_MIGRATED		0x4	// simple password was copied to password

NMASAPI nint32	MAF_SetPassword(MAF_Handle mh, size_t passwordLen, unicode *password, nuint32 setFlags);
NMASAPI nint32	MAF_GetPassword(MAF_Handle mh, size_t *passwordLen, unicode *password, nuint32 *infoFlags, nint32 *remainingLifetime, nint32 *remainingGraceLogins);
NMASAPI nint32	MAF_GetPasswordEx(MAF_Handle mh, nuint32 getFlags, size_t *passwordLen, unicode *password, nuint32 *infoFlags, nint32 *remainingLifetime, nint32 *remainingGraceLogins);

NMASAPI nint32	MAF_PolicyCheck(MAF_Handle mh, size_t passwordLen, unicode *password);
NMASAPI nint32	MAF_AllowPasswordSet (MAF_Handle mh);

NMASAPI nint32	MAF_TraceEnabled(MAF_Handle mh);
NMASAPI nint32	MAF_Trace(MAF_Handle mh, char *message);
NMASAPI nint32	MAF_TraceOnError(MAF_Handle mh, int err, char *message);

#endif
	 
