/*******************************************************************************
  Copyright (c) 2000-2007 Novell, Inc.  All Rights Reserved.                      

  THIS WORK IS  SUBJECT  TO  U.S.  AND  INTERNATIONAL  COPYRIGHT  LAWS  AND
  TREATIES.   NO  PART  OF  THIS  WORK MAY BE  USED,  PRACTICED,  PERFORMED
  COPIED, DISTRIBUTED, REVISED, MODIFIED, TRANSLATED,  ABRIDGED, CONDENSED,
  EXPANDED,  COLLECTED,  COMPILED,  LINKED,  RECAST, TRANSFORMED OR ADAPTED
  WITHOUT THE PRIOR WRITTEN CONSENT OF NOVELL, INC. ANY USE OR EXPLOITATION
  OF THIS WORK WITHOUT AUTHORIZATION COULD SUBJECT THE PERPETRATOR TO
  CRIMINAL AND CIVIL LIABILITY.
 *******************************************************************************/

#ifndef NMAS_TRANSPORT_H
#define NMAS_TRANSPORT_H

/* WARNING:  This header does not support 64-bit development 
*
*  Windows:  _WIN64
*  Sun:      _LP64
*  Linux:    __LP64__  (GNUC)
*  HPUX:     __LP64__ 
*  AIX:      __64BIT__
*/
#if (defined(_WIN64) || defined(_LP64) || defined(__LP64__) || defined (__64BIT__))
#error Header does not support 64-bit development
#endif  


#include <ntypes.h>
#include <unicode.h>
#include <nmasapi.h>


/* NMAS FEATURE FLAGS */
#define NMAS_FEAT_RESERVED1               0x00000001
#define NMAS_FEAT_RESERVED2               0x00000002
#define NMAS_FEAT_RESERVED4               0x00000004
#define NMAS_FEAT_DISCONNECTED            0x00000008


/* NMAS OPTION FLAGS */
#define NMAS_OPT_RESERVED1                0x00000001
#define NMAS_OPT_RESERVED2                0x00000002
#define NMAS_OPT_RESERVED4                0x00000004
#define NMAS_OPT_NO_CRYPTO                0x00000008
#define NMAS_OPT_WRAPPING_KEY_PROVIDED    0x00000010
#define NMAS_OPT_NO_PROMPT_FOR_PWD        0x00000020
#define NMAS_OPT_LOGIN_ONLY               0x00000040
#define NMAS_OPT_NO_METHOD_UI             0x00000080
#define NMAS_OPT_VERIFY                   0x00000100


/* !!!! OBSOLETE FLAGS - USE NEW FLAGS/OPTIONS DEFINED ABOVE !!!!           */
/* These flags defined for backwards compatibility                          */
#define NMAS_TRANSPORT_IS_DISCONNECTED			         NMAS_FEAT_DISCONNECTED
#define NMAS_TRANSPORT_OPTION_NO_CRYPTO			      NMAS_OPT_NO_CRYPTO
#define NMAS_TRANSPORT_OPTION_WRAPPING_KEY_PROVIDED   NMAS_OPT_WRAPPING_KEY_PROVIDED
#define NMAS_TRANSPORT_OPTION_NO_PROMPT_FOR_PASSWORD  NMAS_OPT_NO_PROMPT_FOR_PWD


typedef struct
{
	int		connHandle;
	int		wrappingKey;
	int		niciCtx;
	int		loginWasSuccessful;

	unicode *userName;
	unicode	*treeName;
	int		*authHandle;
}NCP_CONN_INFO;

// NMAS client login user Information
typedef struct NMAS_ClientLoginRequestInfo
{
	unicode *userName;           //required
	unicode *reqSequence;		 //required unless user has a default login sequence
	unicode *treeName;	         //optional
	unicode *reqClearance;		 //optional
	unicode *password;           //optional
} NMAS_ClientLoginRequestInfo;


//NMAS transport call back function
typedef int (N_CALLBACK *NMAS_ClientLoginTransportFcn)(
	int handle,
	void *instructionBuf, size_t instructionLen,
	void *resultBuf, size_t resultLen );

//NMAS_ClientLogin
NMASAPI int NMAS_ClientLogin(
	NMAS_ClientLoginRequestInfo *rInfo,
	NMAS_ClientLoginTransportFcn transportFcn,
	int transportHandle,
	long features, long options, void *reserved1, void *reserved2);

#endif

