/*****************************************************************************

  %name: nmasc32.h %
  %version: idc_nmas#21 %
  $Copyright:

  Copyright (c) 1999 - 2012 Novell, Inc.  All Rights Reserved.                      

  THIS WORK IS  SUBJECT  TO  U.S.  AND  INTERNATIONAL  COPYRIGHT  LAWS  AND
  TREATIES.   NO  PART  OF  THIS  WORK MAY BE  USED,  PRACTICED,  PERFORMED
  COPIED, DISTRIBUTED, REVISED, MODIFIED, TRANSLATED,  ABRIDGED, CONDENSED,
  EXPANDED,  COLLECTED,  COMPILED,  LINKED,  RECAST, TRANSFORMED OR ADAPTED
  WITHOUT THE PRIOR WRITTEN CONSENT OF NOVELL, INC. ANY USE OR EXPLOITATION
  OF THIS WORK WITHOUT AUTHORIZATION COULD SUBJECT THE PERPETRATOR TO
  CRIMINAL AND CIVIL LIABILITY.$

 ****************************************************************************/


#ifndef  _NMASC32_H
#define  _NMASC32_H

/**************************************************************************
 
 These routines are for use by Novell Client32 and only available on Win32
 platforms.  Novell NWSDK dependencies.
  
 The APIs are exported from nmasncp.dll
***************************************************************************/


N_EXTERN_LIBRARY (nint) NMAS_C32Version(
   nuint32         *pMajor,
   nuint32         *pMinor,
   nuint32         *pRevision,
   nuint32         *pBuild);

N_EXTERN_LIBRARY (nint) NMAS_C32Login(
   nstr8          *pTreeName,
   nstr8          *pUserDN,
   nstr8          *pPwd, 
   nstr8          *pServer,
   nstr8          *pSequence,
   nstr8          *pClearance,
   nptr           uiHandle,
   nuint32        uiTimeout,
   nuint32        options,
   nuint32        *pAuthHandle,
   NWCONN_HANDLE  *pConn);
   
N_EXTERN_LIBRARY (nint) NMAS_C32UnlockWorkstation(
   nstr8    *pTreeName,
   nstr8    *pUserDN,
   nstr8    *pPwd,
   nstr8    *pSequence,
   nptr      uiHandle,
   nuint32   uiTimeout,
   nuint32   options); 

N_EXTERN_LIBRARY (nint) NMAS_C32GetErrorMsg(
   int       nmasError,
   nstr8    *pErrMsg,
   nuint32  *pMsgSize);



/**************************************************************************
   NMAS password policy support 
***************************************************************************/

   /* Universal Password synchronization options
   */
#define NMAS_UP_REMOVE_NDS_PWD   0x00000001 // remove NDS pwd when UP is set      
#define NMAS_UP_SYNC_NDS_PWD     0x00000002 // sync UP to NDS password 
#define NMAS_UP_SYNC_SIMPLE_PWD  0x00000004 // sync UP to Simple password
#define NMAS_UP_SYNC_DIST_PWD    0x00000008 // sync UP to distribution pwd
#define NMAS_UP_ADMIN_RETRIEVAL  0x00000010 // allow admin to retrieve pwd
#define NMAS_UP_SELF_RETRIEVAL   0x00000020 // allow self to retrieve pwd

   /* password policy configuration flags
   */
#define NMAS_PWDPOL_UP           0x00000001  // Universal pwd enabled
#define NMAS_PWDPOL_ADV_RULES    0x00000002  // Advanced pwd rules enabled
#define NMAS_PWDPOL_FOR_PWD      0x00000004  // forgotten pwd support enabled
#define NMAS_PWDPOL_FOR_CHK      0x00000008  // check forgotten config at login
#define NMAS_PWDPOL_LOGIN_CHK    0x00000010  // verify pwd matches policy at login

   /* universal password status flags
   */
#define NMAS_UP_ENABLED          0x00000001  // UP enabled
#define NMAS_UP_SET              0x00000002  // UP set
#define NMAS_UP_HISTORY_FULL     0x00000004  // UP History is full
#define NMAS_UP_MATCHES_NDS      0x00000010  // UP matches NDS pwd
#define NMAS_UP_OLDER_THAN_NDS   0x00000020  // UP older than NDS pwd
#define NMAS_UP_MATCHES_SPWD     0x00000040  // UP matches simple pwd


   /* Password configuration settings
   */
#define NMAS_PWD_POLICY_CFG_V1      0

typedef struct _NMAS_PwdPolicyCfg
{
   nuint32 version;
   pnstr8  policyDN;        // password policy DN (dot delimited)

   nuint32 policyFlags;     // pwd policy configuration flags
   nuint32 pwdSyncFlags;    // Universal Password synchronization flags
   
   pnstr8  challengeSetDN;  // forgotten pwd challenge set DN (dot delimited)
   pnstr8  challengeSetID;  // forgotten pwd challenge set ID

}NMAS_PwdPolicyCfg, *pNMAS_PwdPolicyCfg;


#define NMAS_NDS_PWD_POLICY_V1      0

typedef struct _NMAS_NDSPwdPolicy
{
   nuint32 version;
   nuint32 allowUserChange;      // T/F - pwd change allowed
   nuint32 uniqueRequired;       // T/F - password must be unique
   nuint32 passwordRequired;     // T/F - password is required
   nuint32 minLength;            // minimum number of characters in pwd

   nuint32 graceLogins;          // number of grace logins allowed
   nuint32 expirationInterval;   // number of days between pwd expirations

}NMAS_NDSPwdPolicy, *pNMAS_NDSPwdPolicy;


   /* NMAS_C32GetAdvPwdPolicy() options
   */
#define NMAS_ADV_CHANGE_MSG         0x00000001  // include pwd change msg

#define NMAS_ADV_PWD_POLICY_V1      0

typedef struct _NMAS_AdvPwdPolicy
{
   nuint32 version;
   nuint32 allowUserChange;      // T/F - change allowed
   nuint32 uniqueRequired;       // T/F - password must be unique
   nuint32 passwordRequired;     // T/F - password is required
   
   nuint32 graceLogins;          // number of grace logins allowed
   nuint32 expirationInterval;   // number of days between pwd expirations
   nuint32 changeInterval;       // number of days before pwd can be changed
   nuint32 historyLimit;         // number of pwd(s) stored in history
   nuint32 historyExpiration;    // number of days a old pwd is stored
   
   nuint32 minLength;            // minimum number of characters
   nuint32 maxLength;            // maximum number of characters
   nuint32 minUpperCase;         // minimum uppercase characters 
   nuint32 maxUpperCase;         // maximum uppercase characters 
   nuint32 minLowerCase;         // minimum lowercase characters 
   nuint32 maxLowerCase;         // maximum lowercase characters 
   nuint32 allowNumeric;         // T/F -  numeric characters are allowed
   nuint32 allowNumericAsFirst;  // T/F -  first character can be numeric
   nuint32 allowNumericAsLast;   // T/F -  last character can be numeric
   nuint32 minNumeric;           // minimum numeric characters required
   nuint32 maxNumeric;           // maximum numeric characters allowed
   nuint32 allowSpecial;         // T/F - non-alphanumeric characters allowed
   nuint32 allowSpecialAsFirst;  // T/F - first character can be non-alphanumeric 
   nuint32 allowSpecialAsLast;   // T/F - last character can be non-alphanumeric 
   nuint32 minSpecial;           // minimum non-alphanumeric characters required
   nuint32 maxSpecial;           // maximum non-alphanumeric characters allowed
   nuint32 allowExtended;        // T/F - extended characters allowed ( > 0x7F )
   nuint32 maxRepeated;          // maximum times a character can appear
   nuint32 maxConsecutive;       // maximum times a character can appear consecutively
   nuint32 minUnique;            // minimum different characters
   
   pnstr8  changeMsg;            // administrator defined pwd change message 

}NMAS_AdvPwdPolicy, *pNMAS_AdvPwdPolicy;

   /* forgotten password actions
   */
#define NMAS_FOR_ACTION_EMAIL_HINT     1
#define NMAS_FOR_ACTION_SHOW_HINT      2
#define NMAS_FOR_ACTION_CHANGE_PWD     3
#define NMAS_FOR_ACTION_EMAIL_PWD      4

   /* forgotten password policy
   */
#define NMAS_FOR_PWD_POLICY_V1         0

typedef struct _NMAS_ForPwdPolicy
{
   nuint32 version;
   nuint32 authRequired;   // T/F - authentication required before doing action
   nuint32 action;         // forgotten action - see defines 
   pnstr8  loginSeq;       // nmas login sequence for authentication

}NMAS_ForPwdPolicy, *pNMAS_ForPwdPolicy;


N_EXTERN_LIBRARY (nint) NMAS_C32GetPwdPolicyCfg(
   nstr8              *pTreeName, 
   nstr8              *pUserDN,
   nuint32             options,       /* currently 0 */
   nuint32             reqVersion,  /* requested version of policy */
   pNMAS_PwdPolicyCfg *pPwdPolicyCfg);

N_EXTERN_LIBRARY (nint) NMAS_C32GetNDSPwdPolicy(
   nstr8              *pTreeName, 
   nstr8              *pUserDN,
   nuint32             options,       /* currently 0 */
   nuint32             reqVersion,  /* requested version of policy */
   pNMAS_NDSPwdPolicy *pPwdPolicy);

N_EXTERN_LIBRARY (nint) NMAS_C32GetAdvPwdPolicy(
   nstr8              *pTreeName, 
   nstr8              *pPolicyDN,
   nuint32             options,
   nuint32             reqVersion,  /* requested version of policy */
   pNMAS_AdvPwdPolicy *pPwdPolicy);

N_EXTERN_LIBRARY (nint) NMAS_C32GetForgottenPwdPolicy(
   nstr8              *pTreeName, 
   nstr8              *pPolicyDN,
   nuint32             options,       /* currently 0 */
   nuint32             reqVersion,  /* requested version of policy */
   pNMAS_ForPwdPolicy *pPwdPolicy);


N_EXTERN_LIBRARY (nint) NMAS_C32FreePwdPolicyCfg(
   NMAS_PwdPolicyCfg *pPwdPolicyCfg);

N_EXTERN_LIBRARY (nint) NMAS_C32FreeNDSPwdPolicy(
   NMAS_NDSPwdPolicy *pPolicy);

N_EXTERN_LIBRARY (nint) NMAS_C32FreeAdvPwdPolicy(
   NMAS_AdvPwdPolicy *pPolicy);

N_EXTERN_LIBRARY (nint) NMAS_C32FreeForgottenPwdPolicy(
   NMAS_ForPwdPolicy *pPolicy);


N_EXTERN_LIBRARY (nint) NMAS_C32CheckPwd( 
   nstr8           *pTreeName, 
   nstr8           *pUserDN,
   nstr8           *pPwd); /* local code page - pass NULL to check current pwd */

N_EXTERN_LIBRARY (nint) NMAS_C32GetPwd( 
   nstr8           *pTreeName, 
   nstr8           *pUserDN,
   nuint32         *pPwdLen,
   nstr8           *pPwd );  /* local code page */

N_EXTERN_LIBRARY (nint) NMAS_C32PwdStatus( 
   nstr8           *pTreeName, 
   nstr8           *pUserDN,
   nuint32         *pPwdStatus);

N_EXTERN_LIBRARY (nint) NMAS_C32DeletePwd( 
   nstr8           *pTreeName, 
   nstr8           *pUserDN);

N_EXTERN_LIBRARY (nint) NMAS_C32SetPwd( 
   nstr8           *pTreeName, 
   nstr8           *pUserDN,
   nstr8           *pNewPwd );  /* local code page */

N_EXTERN_LIBRARY (nint) NMAS_C32ChangePwd( 
   nstr8           *pTreeName, 
   nstr8           *pUserDN,
   nstr8           *pOldPwd,   /* local code page */
   nstr8           *pNewPwd ); /* local code page */

N_EXTERN_LIBRARY(nint) NMAS_C32SetPwdHint(
   nstr8              *pTreeName, 
   nstr8              *pUserDN,
   nuint32             msgSize,
   nstr8              *pHintMsg);

N_EXTERN_LIBRARY(nint) NMAS_C32GetPwdHint(
   nstr8              *pTreeName, 
   nstr8              *pUserDN,
   nuint32            *pMsgSize,
   nstr8              *pHintMsg);

N_EXTERN_LIBRARY(nint) NMAS_C32SetPwdReminder(
   nstr8              *pTreeName, 
   nstr8              *pUserDN,
   nuint32             msgSize,
   nstr8              *pReminderMsg);

N_EXTERN_LIBRARY(nint) NMAS_C32GetPwdReminder(
   nstr8              *pTreeName, 
   nstr8              *pUserDN,
   nuint32            *pMsgSize,
   nstr8              *pReminderMsg);

   /*  Challenge Response
   */

   /* required types 
   */
#define NMAS_CR_RANDOM_QUESTION     1
#define NMAS_CR_REQUIRED_QUESTION   2

   /* question types 
   */
#define NMAS_CR_USER_DEFINED        3
#define NMAS_CR_ADMIN_DEFINED       4

typedef struct _NMAS_CRType 
{
   pnstr8    challenge;    // Challenge question
   nuint32   chalSize;     // Question length
   pnstr8    response;     // Challenge response
   nuint32   respSize;     // Response length
   nint32    minSize;      // Minimum size for the response
   nint32    maxSize;      // Maximum size for the response
   nuint32   type;         // Admin Defined or User Defined
   nuint32   required;     // Required Challenge or Random Challenge
   pnstr8    GUID;         // GUID of the Challenge set
} NMAS_CRType, *pNMAS_CRType;


N_EXTERN_LIBRARY(nint) NMAS_C32GetChalResp(
   nstr8             *pTreeName, 
   nstr8             *pUserDN,
   NMAS_CRType      **cr, 
   nint32            *crCount
);

N_EXTERN_LIBRARY(nint) NMAS_C32SetChalResp(
   nstr8             *pTreeName, 
   nstr8             *pUserDN,
   NMAS_CRType       *cr, 
   nint32            crCount
);

N_EXTERN_LIBRARY(nint) NMAS_C32ChalRespFree(
   NMAS_CRType     **cr,    // Array  of admin Defined Challenge/Response sets
   nint32            crCount     // Number of admin defined Challenge/Response sets
);



/**************************************************************************
   NMAS sequence / clearance support 
***************************************************************************/

   /* graded authentication clearance range information
   */
#define NMAS_MAX_LABEL_LEN  32

typedef struct _NMAS_GARange
{
   nuint32     rangeType;
   nuint32     readLabelType;
   nstr8       readLabelName[NMAS_MAX_LABEL_LEN];
   nuint32     writeLabelType;
   nstr8       writeLabelName[NMAS_MAX_LABEL_LEN];

} NMAS_GARange, *pNMAS_GARange;


typedef struct _NMAS_List 
{
   nstr8    *pDefault;      /* default item in list - NULL if no default */
   nuint     count;         /* number of ptrs in list  */
   pnstr8   *pList;         /* array of pointers */

}  NMAS_List, *pNMAS_List;


N_EXTERN_LIBRARY (nint) NMAS_C32GetSeqList(
   nstr8       *pTreeName,
   nstr8       *pUserDN,
   nuint32      options,      /* reserved must be 0 */
   NMAS_List   *pSeqList);

N_EXTERN_LIBRARY (nint) NMAS_C32GetClearList(
   nstr8       *pTreeName,
   nstr8       *pUserDN,
   nuint32      options,      /* reserved must be 0 */
   NMAS_List   *pClearList);

N_EXTERN_LIBRARY (nint) NMAS_C32FreeList(
   NMAS_List   *pList);

N_EXTERN_LIBRARY (nint) NMAS_C32GetRange(
   NWCONN_HANDLE  conn,
   NMAS_GARange   *pRange);



/**************************************************************************
   Identity Plugin support  
***************************************************************************/

   /* ID Plugin operational flags 
   */
#define ID_PLUGIN_AUTO_RESTART    0x00000001  /* restart plugin after failure */
#define ID_PLUGIN_AUTO_LOGIN      0x00000002  /* start login process after ID plugin as returned */

   /* ID Plugin user interface flags
   */
#define ID_PLUGIN_HIDE_OK         0x00000001
#define ID_PLUGIN_HIDE_CANCEL     0x00000002
#define ID_PLUGIN_HIDE_USERNAME   0x00000004
#define ID_PLUGIN_HIDE_PASSWORD   0x00000008


#define ID_PLUGIN_DATA_V1         1

   /* passed to callback routine
   */
typedef struct _NMAS_PluginData 
{
   int      version;
   nstr8    *pUserName;
   nstr8    *pTree;
   nstr8    *pServer;
   nstr8    *pContext;
   nstr8    *pSequence;
   nstr8    *pClearance;

}  NMAS_PluginData, *pNMAS_PluginData;


   /* Callback function type - passed to NMAS_C32InitPlugin
   */
typedef nint (N_API *pFunc_PluginCallback)(
   NMAS_PluginData    *pData);


typedef struct _PluginHandle
{
   nuint32   opFlags; 
   nuint32   uiFlags;
   nstr8    *promptMsg;
   nstr8    *waitMsg;
   void     *ctrInfo;

} PluginHandle, *pPluginHandle;

   /* valid mode flags
   */
#define IDPLUGIN_MODE_NETWORK		0x00000001
#define IDPLUGIN_MODE_LOCAL			0x00000002
#define IDPLUGIN_MODE_LDAP			0x00000004

   /* initializes the plugin (loads library, etc)
   */
N_EXTERN_LIBRARY (nint) NMAS_C32InitPlugin( 
   pFunc_PluginCallback  callback,   /* ptr to callback function */
   nuint32               options,    /* reserved must be 0 */
   pPluginHandle        *ppH);       /* out - plugin handle ptr */

N_EXTERN_LIBRARY (nint) NMAS_C32StartPlugin(
   PluginHandle *pH);

N_EXTERN_LIBRARY (nint) NMAS_C32StopPlugin(
   PluginHandle *pH);

   /* frees PluginHandle resources (unload library, etc)
   */
N_EXTERN_LIBRARY (nint) NMAS_C32DeinitPlugin(
   PluginHandle *pH);





/**************************************************************************
   Function ptr typedefs - the client dynamically loads the library and calls
   the APIs through function ptrs.
***************************************************************************/

typedef nint (N_API *pFunc_NMAS_C32Version)(
   nuint32         *major,
   nuint32         *minor,
   nuint32         *revision,
   nuint32         *build);

typedef nint (N_API *pFunc_NMAS_C32FreeList)(
   NMAS_List   *pList);

typedef nint (N_API *pFunc_NMAS_C32GetClearList)(
   nstr8       *pTreeName,
   nstr8       *pUserDN,
   nuint32      options,    
   NMAS_List   *pClearList);

typedef nint (N_API *pFunc_NMAS_C32GetSeqList)(
   nstr8       *pTreeName,
   nstr8       *pUserDN,
   nuint32      options,    
   NMAS_List   *pSeqList);

typedef nint (N_API *pFunc_NMAS_C32GetRange)(
   NWCONN_HANDLE  conn,
   NMAS_GARange   *pRange);

typedef nint (N_API *pFunc_NMAS_C32GetErrorMsg)(
   int       nmasError,
   nstr8    *pErrMsg,
   nuint32  *pMsgSize);

typedef nint (N_API *pFunc_NMAS_C32UnlockWorkstation)(
   nstr8    *pTreeName,
   nstr8    *pUserDN,
   nstr8    *pPwd,
   nstr8    *pSequence,
   nptr      uiHandle,
   nuint32   uiTimeout,
   nuint32   options); 

typedef nint (N_API *pFunc_NMAS_C32Login)(
   nstr8          *pTreeName,
   nstr8          *pUserDN,
   nstr8          *pPwd, 
   nstr8          *pServer,
   nstr8          *pSequence,
   nstr8          *pClearance,
   nptr           uiHandle,
   nuint32        uiTimeout,
   nuint32        options,
   nuint32        *pAuthHandle,
   NWCONN_HANDLE  *pConn);

typedef nint (N_API *pFunc_NMAS_C32GetPwdPolicyCfg)(
   nstr8              *pTreeName, 
   nstr8              *pUserDN,
   nuint32             options,     
   nuint32             reqVersion,
   pNMAS_PwdPolicyCfg *pPwdPolicyCfg);

typedef nint (N_API *pFunc_NMAS_C32GetNDSPwdPolicy)(
   nstr8              *pTreeName, 
   nstr8              *pUserDN,
   nuint32             options,     
   nuint32             reqVersion,
   pNMAS_NDSPwdPolicy *pPwdPolicy);

typedef nint (N_API *pFunc_NMAS_C32GetAdvPwdPolicy)(
   nstr8              *pTreeName, 
   nstr8              *pPolicyDN,
   nuint32             options,
   nuint32             reqVersion,
   pNMAS_AdvPwdPolicy *pPwdPolicy);

typedef nint (N_API *pFunc_NMAS_C32GetForgottenPwdPolicy)(
   nstr8              *pTreeName, 
   nstr8              *pPolicyDN,
   nuint32             options,  
   nuint32             reqVersion,
   pNMAS_ForPwdPolicy *pPwdPolicy);

typedef nint (N_API *pFunc_NMAS_C32FreePwdPolicyCfg)(
   NMAS_PwdPolicyCfg *pPwdPolicyCfg);

typedef nint (N_API *pFunc_NMAS_C32FreeNDSPwdPolicy)(
   NMAS_NDSPwdPolicy *pPolicy);

typedef nint (N_API *pFunc_NMAS_C32FreeAdvPwdPolicy)(
   NMAS_AdvPwdPolicy *pPolicy);

typedef nint (N_API *pFunc_NMAS_C32FreeForgottenPwdPolicy)(
   NMAS_ForPwdPolicy *pPolicy);

typedef nint (N_API *pFunc_NMAS_C32CheckPwd)( 
   nstr8           *pTreeName, 
   nstr8           *pUserDN,
   nstr8           *pPwd);    

typedef nint (N_API *pFunc_NMAS_C32GetPwd)( 
   nstr8           *pTreeName, 
   nstr8           *pUserDN,
   size_t          *pPwdLen,
   nstr8           *pPwd );       /* local code page */

typedef nint (N_API *pFunc_NMAS_C32PwdStatus)( 
   nstr8           *pTreeName, 
   nstr8           *pUserDN,
   nuint32         *pwdStatus);

typedef nint (N_API *pFunc_NMAS_C32DeletePwd)( 
   nstr8           *pTreeName, 
   nstr8           *pUserDN);

typedef nint (N_API *pFunc_NMAS_C32SetPwd)( 
   nstr8           *pTreeName, 
   nstr8           *pUserDN,
   nstr8           *pNewPwd );       /* local code page */

typedef nint (N_API *pFunc_NMAS_C32ChangePwd)( 
   nstr8           *pTreeName, 
   nstr8           *pUserDN,
   nstr8           *pOldPwd,         /* local code page */
   nstr8           *pNewPwd );       /* local code page */

typedef nint (N_API *pFunc_NMAS_C32SetPwdHint)(
   nstr8              *pTreeName, 
   nstr8              *pUserDN,
   nuint32             msgSize,
   nstr8              *pHintMsg);

typedef nint (N_API *pFunc_NMAS_C32GetPwdHint)(
   nstr8              *pTreeName, 
   nstr8              *pUserDN,
   nuint32            *pMsgSize,
   nstr8              *pHintMsg);

typedef nint (N_API *pFunc_NMAS_C32SetPwdReminder)(
   nstr8              *pTreeName, 
   nstr8              *pUserDN,
   nuint32             msgSize,
   nstr8              *pHintMsg);

typedef nint (N_API *pFunc_NMAS_C32GetPwdReminder)(
   nstr8              *pTreeName, 
   nstr8              *pUserDN,
   nuint32            *pMsgSize,
   nstr8              *pHintMsg);

typedef nint (N_API *pFunc_NMAS_C32InitPlugin)( 
   pFunc_PluginCallback  callback, 
   nuint32               options,  
   pPluginHandle        *ppH);     

typedef nint (N_API *pFunc_NMAS_C32StartPlugin)(
   PluginHandle *pH);

typedef nint (N_API *pFunc_NMAS_C32StopPlugin)(
   PluginHandle *pH);

typedef nint (N_API *pFunc_NMAS_C32DeinitPlugin)(
   PluginHandle *pH);

typedef nint (N_API *pFunc_NMAS_C32GetChalResp)(
   nstr8             *pTreeName, 
   nstr8             *pUserDN,
   NMAS_CRType      **cr, 
   nint32            *crCount);

typedef nint (N_API *pFunc_NMAS_C32SetChalResp)(
   nstr8             *pTreeName, 
   nstr8             *pUserDN,
   NMAS_CRType       *cr, 
   nint32            crCount);

typedef nint (N_API *pFunc_NMAS_C32ChalRespFree)(
   NMAS_CRType     **cr,
   nint32            crCount);

#endif
