/*****************************************************************************

  %name: nmasconf.h %
  %version: idc_nmas#14 %
  $Copyright:

  Copyright (c) 1999 - 2007 Novell, Inc.  All Rights Reserved.                      

  THIS WORK IS  SUBJECT  TO  U.S.  AND  INTERNATIONAL  COPYRIGHT  LAWS  AND
  TREATIES.   NO  PART  OF  THIS  WORK MAY BE  USED,  PRACTICED,  PERFORMED
  COPIED, DISTRIBUTED, REVISED, MODIFIED, TRANSLATED,  ABRIDGED, CONDENSED,
  EXPANDED,  COLLECTED,  COMPILED,  LINKED,  RECAST, TRANSFORMED OR ADAPTED
  WITHOUT THE PRIOR WRITTEN CONSENT OF NOVELL, INC. ANY USE OR EXPLOITATION
  OF THIS WORK WITHOUT AUTHORIZATION COULD SUBJECT THE PERPETRATOR TO
  CRIMINAL AND CIVIL LIABILITY.$

 ****************************************************************************/

#ifndef _NMASCONF_H
#define _NMASCONF_H


   /* Maximum sizes of client/method information strings
   */
#define NMAS_VERSION_LEN       24
#define NMAS_BUILD_DATE_LEN    24
#define NMAS_DESC_LEN          64
#define NMAS_MODULE_LEN        256
#define NMAS_FUNC_LEN          64
#define NMAS_VENDOR_LEN        64
#define NMAS_MSG_LEN           256



/**************************************************************************
  NMAS client version information
***************************************************************************/   
   
   /* client information structure
   */
typedef struct _NMAS_Info
{
   char  version[NMAS_VERSION_LEN];
   char  build_date[NMAS_BUILD_DATE_LEN];
   char  description[NMAS_DESC_LEN];

} NMAS_Info, *pNMAS_Info;

   /* queries client build and version information
   */
NMASAPI int NMAS_ClientInfo(
   int             *major,
   int             *minor,
   int             *revision,
   int             *build,
   NMAS_Info       *info);

typedef  int (*pFunc_NMAS_ClientInfo)(
   int             *major,
   int             *minor,
   int             *revision,
   int             *build,
   NMAS_Info       *info);



/*************************************************************************
   NMAS client configuration  
*************************************************************************/

   /* NMAS_ClientConfig mode options 
   */
#define NMAS_CONFIG_CREATE    1     
#define NMAS_CONFIG_REMOVE    2     
#define NMAS_CONFIG_UPDATE    3     


   /* performs initial client configuration steps
   */  
NMASAPI int NMAS_ClientConfigure( 
   int mode  );

typedef  int (*pFunc_NMAS_ClientConfigure)( 
   int mode  );

   

/**************************************************************************
  NMAS Client method configuration 
***************************************************************************/   
   
   /* method information structure
   */
typedef struct _NMAS_MethodInfo
{
   int   methodID;                             
   char  module[NMAS_MODULE_LEN];       
   char  vendor[NMAS_VENDOR_LEN];
   char  version[NMAS_VERSION_LEN];
   char  build_date[NMAS_BUILD_DATE_LEN];
   char  description[NMAS_DESC_LEN];
   char  network_func[NMAS_FUNC_LEN];        // pFunc_Method function
   char  local_func[NMAS_FUNC_LEN];          // pFunc_Method function
   char  info_func[NMAS_FUNC_LEN];           // pFunc_MethodInfo function

} NMAS_MethodInfo, *pNMAS_MethodInfo;
   

   /* Methods supporting dynamic configuration must export and an 
    * "info_func" API matching this signature.   
   */
typedef int (*pFunc_MethodInfo)(NMAS_MethodInfo *info);

   /* Methods must export and "network_func" and optionally a "local_func" 
    * matching this signature
   */
typedef int (*pFunc_Method)(MAF_Handle mh, unsigned long param);


   /* queries client method configuration information
   */
NMASAPI int NMAS_ClientMethodInfo( 
   int methodID, 
   NMAS_MethodInfo *info);

typedef  int (*pFunc_NMAS_ClientMethodInfo)( 
   int methodID, 
   NMAS_MethodInfo *info);


   /* enumerates client methods in configuration - index should be  
    * initialized to 0 and incremented with each call.  Returns
    * NMAS_E_CFG_NO_MORE_ITEMS when complete.
   */
NMASAPI int NMAS_ClientMethodEnum( 
   int index, 
   NMAS_MethodInfo *info);

typedef  int (*pFunc_NMAS_ClientMethodEnum)( 
   int index, 
   NMAS_MethodInfo *info);


   /* adds a method to the client configuration
   */      
NMASAPI int NMAS_ClientAddMethod( 
   NMAS_MethodInfo *info);

typedef  int (*pFunc_NMAS_ClientAddMethod)( 
   NMAS_MethodInfo *info);


   /* removes a method from the client configuration 
   */
NMASAPI int NMAS_ClientRemoveMethod( 
   int methodID);

typedef  int (*pFunc_NMAS_ClientRemoveMethod)( 
   int methodID);



/**************************************************************************
  Identity plugin configuration
***************************************************************************/   
   
   /* Identity Plugin information structure
   */
typedef struct _NMAS_PluginInfo
{
   int   type;
   char  module[NMAS_MODULE_LEN];       
   char  vendor[NMAS_VENDOR_LEN];
   char  version[NMAS_VERSION_LEN];
   char  build_date[NMAS_BUILD_DATE_LEN];
   char  description[NMAS_DESC_LEN];

   char  network_reg_func[NMAS_FUNC_LEN];    // pFunc_Register
   char  local_reg_func[NMAS_FUNC_LEN];      // pFunc_Register 
   char  info_func[NMAS_FUNC_LEN];           // pFunc_PluginInfo 

   char  prompt_msg[NMAS_MSG_LEN];
   char  wait_msg[NMAS_MSG_LEN];
   int   op_flags;
   int   ui_flags;

} NMAS_PluginInfo, *pNMAS_PluginInfo;
   
   /* Identity Plugins supporting dynamic configuration must export and an 
    * "info_func" API matching this signature.   
   */
typedef  int (*pFunc_PluginInfo)(NMAS_PluginInfo *info);

   /* op_flags:  These flags are not validated by the nmas client.  The meaning  
    * of a flag is dependent upon the UI component which loads/runs the plugin.  
    * The following flags are defined for backwards compatibility with 
    * Win32 Client32 environments.
   */
#define ID_PLUGIN_AUTO_RESTART   0x00000001    
#define ID_PLUGIN_AUTO_LOGIN     0x00000002    

   /* ui_flags:  These flags are not validated by the nmas client.  The meaning 
    * of a flag is dependent upon the UI component which loads/runs the plugin.  
    * The following flags are defined for backwards compatibility with 
    * Win32 Client32 environments.
   */
#define ID_PLUGIN_HIDE_OK        0x00000001
#define ID_PLUGIN_HIDE_CANCEL    0x00000002
#define ID_PLUGIN_HIDE_USERNAME  0x00000004

   /* ID_PLUGIN_TYPE1 - Plugins 
    *
    * Type1 ID plugins export 3 APIs (register, start, stop) matching the
    * signatures below.  UI components which use type1 plugins must have 
    * a set_identity function matching the signature below. 
    *
    * The UI component loads the plugin and calls the register function.  The
    * ID Plugin's register function stores the UI component's "set_identity"
    * callback and returns it's "start" and "stop" function ptrs.
   */
#define ID_PLUGIN_TYPE1          1

   /* validFields flags
   */
#define NLI_FIELD_USERNAME	   0x00000001
#define NLI_FIELD_CONTEXT	   0x00000002
#define NLI_FIELD_TREE		   0x00000004
#define NLI_FIELD_SERVER	   0x00000008
#define NLI_FIELD_SEQUENCE	   0x00000010
#define NLI_FIELD_CLEARANCE	0x00000020
#define NLI_FIELD_LDAP_DN	   0x00000040

   /* valid operation flags
   */
#define IDPLUGIN_MODE_NETWORK		0x00000001
#define IDPLUGIN_MODE_LOCAL			0x00000002
#define IDPLUGIN_MODE_LDAP			0x00000004

typedef struct _NMAS_LOGIN_IDENTITY
{
	int		size;
	int		validFields;
	wchar_t	userName[256];
	wchar_t  context[256];
	wchar_t  tree[256];
	wchar_t  server[256];
	wchar_t  sequence[256];
	wchar_t  clearance[256];
	wchar_t  ldapDN[256];

} NMAS_LOGIN_IDENTITY, *pNMAS_LOGIN_IDENTITY;
   
   /* UI Components pass a function ptr matching this type to the plugin
    * register function
   */
typedef  int (*pFunc_SetIdentity)(NMAS_LOGIN_IDENTITY *info);

   /* type1 plugin exported api signatures
   */
typedef  int (*pFunc_StartPlugin)();
typedef  int (*pFunc_StopPlugin)();
typedef  int (*pFunc_RegisterPlugin)(pFunc_SetIdentity  pfunc_Callback, 
                                     pFunc_StartPlugin *pfunc_Start,
                                     pFunc_StopPlugin  *pfunc_Stop);


   /* adds an identity plugin to the client configuration
   */
NMASAPI int NMAS_ClientAddPlugin(
   NMAS_PluginInfo *info);

typedef  int (*pFunc_NMAS_ClientAddPlugin)(
   NMAS_PluginInfo *info);


   /* removes identity plugin from the client configuration
   */
NMASAPI int NMAS_ClientRemovePlugin(void);

typedef  int (*pFunc_NMAS_ClientRemovePlugin)(void);


   /* queries identity plugin configuration information
   */
NMASAPI int NMAS_ClientPluginInfo(
   NMAS_PluginInfo *info);

typedef  int (*pFunc_NMAS_ClientPluginInfo)(
   NMAS_PluginInfo *info);



/**************************************************************************
  Debug Trace configuration
***************************************************************************/   
   
#define NMAS_TRACE_FILE_LEN         256

#define NMAS_TRACE_OFF              0
#define NMAS_TRACE_ON               1

#define NMAS_TRACE_MODE_APPEND      1
#define NMAS_TRACE_MODE_OVERWRITE   2

   /* client trace information structure
   */
typedef struct _NMAS_TraceInfo
{
   int   status;                          // tracing on | off
   int   mode;                            // append | overwrite file
   char  outputFile[NMAS_TRACE_FILE_LEN]; // output file name

} NMAS_TraceInfo, *pNMAS_TraceInfo;


   /* adds trace info to the client configuration
   */
NMASAPI int NMAS_ClientAddTraceInfo(
   NMAS_TraceInfo *info);

typedef  int (*pFunc_NMAS_ClientAddTraceInfo)(
   NMAS_TraceInfo *info);


   /* removes trace info from the client configuration
   */
NMASAPI int NMAS_ClientRemoveTraceInfo(void);

typedef  int (*pFunc_NMAS_ClientRemoveTraceInfo)(void);


   /* queries trace configuration data
   */
NMASAPI int NMAS_ClientTraceInfo(
   NMAS_TraceInfo *info);

typedef  int (*pFunc_NMAS_ClientTraceInfo)(
   NMAS_TraceInfo *info);


/**************************************************************************
  NMAS Language configuration
***************************************************************************/   
   
   /* NOTE: The NMAS Client is localized in several languages.  On MS Vista,
   *        the client installs all localizations and can be configured to
   *        use a specific language.  The Language configuration APIs can be 
   *        used to query the language the client is using.
   *
   *        Methods (LCMs) can use these APIs to query the language and
   *        load the appropriate localized resources.
   *         
   *        All localizations are considered "neutral" with repect to 
   *        locale.  For example, English (en) is available.  There is 
   *        no distinction made between English-US (en-US), 
   *        English-Belize (en-BZ), or English-Austrialia (en-AU)
   *
   *        This support is only available on MS Vista.  The APIs return 
   *        NMAS_E_NOT_SUPPORTED on other platforms.  On other platforms, 
   *        the NMAS Client is installed for a specified language.  The 
   *        language can only be changed by a reinstall.
   */
   
   /* Language ID(s) 
   */
#define NMAS_LANGUAGE_EN            1     // English
#define NMAS_LANGUAGE_FR			   2     // French
#define NMAS_LANGUAGE_DE				3     // German
#define NMAS_LANGUAGE_IT			   4     // Italian 
#define NMAS_LANGUAGE_JA				5     // Japanese
#define NMAS_LANGUAGE_PT			   6     // Portuguese
#define NMAS_LANGUAGE_RU				7     // Russian
#define NMAS_LANGUAGE_ES				8     // Spanish
#define NMAS_LANGUAGE_SV				9     // Swedish
#define NMAS_LANGUAGE_PL			   10    // Polish
#define NMAS_LANGUAGE_ZH			   11    // Chinese 


   /* returns the NMAS Client Language ID
   */
NMASAPI int NMAS_ClientGetLanguage(
   int *langID);

typedef  int (*pFunc_NMAS_ClientGetLanguage)(
   int *langID);

#if defined(NMAS_WIN)
/**************************************************************************
  Method local login configuration
***************************************************************************/   

typedef struct _NMAS_Method_LL_Info
{
   int   methodID;                             
   char  module[NMAS_MODULE_LEN]; 
   char  local_func[NMAS_FUNC_LEN];          // pFunc_NMAS_Method_LL_Info function
   char  disconn_stat_func[NMAS_FUNC_LEN];   
} NMAS_Method_LL_Info, *pNMAS_Method_LL_Info;

   /*
   * Adds the local login configuration for method(methodID)
   */   
NMASAPI int
NMAS_ClientAddMethodLL (NMAS_Method_LL_Info *info);

typedef int (*pFunc_NMAS_AddMethod_LL)(NMAS_Method_LL_Info *info);

   /*
   * Removes the local login configuration for method(methodID)
   */
NMASAPI int
NMAS_ClientRemoveMethodLL(int methodID);

typedef int (*pFunc_NMAS_RemoveMethod_LL)(int methodID);

   /*
   * Gets the method's local login configuration
   */
NMASAPI int
NMAS_ClientMethodLLInfo(int methodID, NMAS_Method_LL_Info *info);

typedef int (*pFunc_NMAS_Method_LL_Info)(int methodID, NMAS_Method_LL_Info *info);

   /*
   * Gets local login configuration for all methods
   */
NMASAPI int
NMAS_ClientMethodLLInfoEnum(int noOfKeys, NMAS_Method_LL_Info** info);

typedef int (*pFunc_NMAS_Method_LL_Info_Enum)(int noOfKeys, NMAS_Method_LL_Info** info);

 /*
   * Gets the count of local login configured methods
   */
NMASAPI int
NMAS_ClientMethodLLCount(int* noOfKeys);

typedef int (*pFunc_NMAS_Method_LL_Count)(int* noOfKeys);

#endif

#endif

