/******************************************************************************

  Copyright (c) 1999-2008 Novell, Inc.  All Rights Reserved.                      

  THIS WORK IS  SUBJECT  TO  U.S.  AND  INTERNATIONAL  COPYRIGHT  LAWS  AND
  TREATIES.   NO  PART  OF  THIS  WORK MAY BE  USED,  PRACTICED,  PERFORMED
  COPIED, DISTRIBUTED, REVISED, MODIFIED, TRANSLATED,  ABRIDGED, CONDENSED,
  EXPANDED,  COLLECTED,  COMPILED,  LINKED,  RECAST, TRANSFORMED OR ADAPTED
  WITHOUT THE PRIOR WRITTEN CONSENT OF NOVELL, INC. ANY USE OR EXPLOITATION
  OF THIS WORK WITHOUT AUTHORIZATION COULD SUBJECT THE PERPETRATOR TO
  CRIMINAL AND CIVIL LIABILITY.

 *****************************************************************************/
#ifndef _NMASPXY_H
#define _NMASPXY_H

#include <nmasapi.h>
#include <nmasaddr.h>

#define INVALID_NDS_CONTEXT -1
 
typedef struct MethodIDsBuffer
{
    nuint32 methodLen;
    nuint32 methodID[4];
} MethodIDsBuffer, *pMethodIDsBuffer;

NMASAPI int NMAS_CreateContext(
    nuint32 *nmasHandle);

NMASAPI int NMAS_DestroyContext(
    nuint32 nmasHandle);

NMASAPI int NMAS_DestroyRemoteClientContext(
    nuint32 nmasHandle);

NMASAPI int NMAS_CanDo(
    nuint32 nmasHandle,
    nuint32 numMethods,
    MethodIDsBuffer *canDoMethods,
    size_t reserved1,
    void *reserved2,
    MethodIDsBuffer *doMethod,
    MAF_Handle *mafHandle);

NMASAPI int NMAS_WhatNext(
    nuint32 nmasHandle,
    MethodIDsBuffer *doMethod,
    MAF_Handle *mafHandle);

NMASAPI int NMAS_InvokeMethod(
    MAF_Handle mafHandle,
    MethodIDsBuffer *method);
    
// Flags for NMAS_AID_OPTIONS
#define NMAS_OPTIONS_LOCAL_ONLY_LOGIN   0x1

NMASAPI int NMAS_PutAttribute(
    nuint32 nmasHandle,
    nuint32 aid,
    void *uniTag,
    nuint32 attributeLenInBytes,
    void *attributeValue);

NMASAPI int NMAS_GetAttribute(
    nuint32 nmasHandle,
    nuint32 aid,
    void *uniTag,
    nuint32 *attributeLenInBytes,
    void *attributeValue);

NMASAPI int NMAS_PutNetworkAddress(
    nuint32 nmasHandle,
    nuint32 addrLenInBytes,     // Must be at least the size of NMAS_NETADDRESS
    NMAS_NETADDRESS *address);

NMASAPI int NMAS_GetNetworkAddress(
    nuint32 nmasHandle,
    nuint32 *addrLenInBytes,
    NMAS_NETADDRESS *networkAddress);

NMASAPI int NMAS_FindLoginSequences(
    nuint32 nmasHandle,
    nint32 *iter,
    nuint32 numMethods,
    MethodIDsBuffer *methods,
    nuint32 bufferLenInBytes,
    void *buffer);

NMASAPI int NMAS_FindMethods(
    unicode *seqName,
    nuint32 *bufferLen,
    MethodIDsBuffer *methods,
    nuint32 *methodCnt);

NMASAPI int NMAS_GetAvailableMethods(
    nuint32 *bufferLen,
    MethodIDsBuffer *methods,
    nuint32 *methodCnt);

NMASAPI int NMAS_GetAvailableLCMs(
    nuint32 *bufferLen,
    MethodIDsBuffer *methods,
    nuint32 *methodCnt);
    
NMASAPI int NMAS_GetAvailableACMs(
    nuint32 *bufferLen,
    MethodIDsBuffer *methods,
    nuint32 *methodCnt);
    
NMASAPI int NMAS_AuthenticateConnection(
    nuint32 nmasHandle,
    int connID);

// Obsolete -- use NMAS_SetLocalIdentity or NMAS_SetLocalIdentityEx instead
NMASAPI int NMAS_LocalAuthenticate(
    nuint32 nmasHandle,
    MAF_Handle unused,
    nint reserved,
    nint ndsContext);

// Obsolete -- use NMAS_SetIdentity instead
NMASAPI int NMAS_Authenticate(
    nuint32 nmasHandle,
    MAF_Handle unused,
    nint reserved,
    nint ndsContext);

NMASAPI int NMAS_SetLocalIdentity(
    nuint32 nmasHandle,
    nint reserved,
    nint ndsContext);

// flags for NMAS_SetLocaIdentityEx and NMASLocalPwdLogin
#define NMAS_LOCAL_ID_LICENSED  0x1
#define NMAS_USE_DEFAULT_GRADE  0x2
NMASAPI int NMAS_SetLocalIdentityEx(
    nuint32 nmasHandle,
    nuint32 flags,
    nint reserved,
    nint ndsContext);
    
// Flags for NMASLocalPwdLogin
#define NMAS_LOCAL_PWD_VERIFY                  0x100
#define NMAS_LOCAL_PWD_DONT_UPDATE_ON_ERROR    0x200
#define NMAS_LOCAL_PWD_DONT_UPDATE_ON_SUCCESS  0x400
NMASAPI int NMAS_LocalPwdLogin(
    nuint32 nmasHandle,
    nuint32 flags,
    unicode *userName,
    unicode *pwd,
    NMAS_NETADDRESS *address,
    nint reserved,
    nuint32 *pwdWarning);
    
// Flags for stringType
#define NMAS_AUTHN_UTF8_STRING      1
#define NMAS_AUTHN_UNICODE_STRING   2

// Flags for userNameType
#define NMAS_AUTHN_USER_DN          1  

// Flags for issuerType
#define NMAS_AUTHN_ISSUER_TREE_NAME    1  

typedef struct _NMAS_AuthNCred
{
   int              stringType;         /* indicates that strings are unicode, UTF-8, or w_char */
   void             *securityDomain;    /* intended security domain (such as tree) */
   void             *userName;          /* intended user name */
   int              userNameType;       /* user name type (such as user DN) */
   void             *issuer;            /* issuer name that issued authentication materials */
   int              issuerType;         /* isser name type (such as tree name) */
   NMAS_GUID        guid;               /* authN credential type */
   MethodIDsBuffer  authNMethod;        /* method identifier associated with the credential */
   nuint32           size;               /* authN credential size (in bytes) */
   void             *data;              /* authN credential */
} NMAS_AuthNCred, *pNMAS_AuthNCred;

NMASAPI int NMAS_GetAuthNCreds(
    nuint32 nmasHandle,
    nuint32 *authNCredCnt,
    NMAS_AuthNCred ***authNCreds);
    
NMASAPI int NMAS_AuthNCredsUpgrade(
    nint reserved,
    nuint32 *newAuthNCredCnt,
    NMAS_AuthNCred ***newAuthNCreds);
    
NMASAPI int NMAS_AuthNCredsRefresh(
    unicode *treeName,
    unicode *userDN,
    nuint32 oldAuthNCredCnt,
    NMAS_AuthNCred **oldAuthNCreds,
    nuint32 *newAuthNCredCnt,
    NMAS_AuthNCred ***newAuthNCreds);
    
//NMAS transport call back function
typedef int (*NMAS_TransportFcn)(
	void *transArg,
	void *instructionBuf,
    nuint32 instructionLen,
	void *resultBuf,
    nuint32 resultLen );

//NMAS close transport call back function
typedef int (*NMAS_CloseTransportFcn)(
	void *transArg);
    
NMASAPI int NMAS_AuthNCredAuthenticate(
    NMAS_TransportFcn transportFcn,
    void *transportArg,
    int options,
    nuint32 oldAuthNCredCnt,
    NMAS_AuthNCred **oldAuthNCreds,
    nuint32 *newAuthNCredCnt,
    NMAS_AuthNCred ***newAuthNCreds,
    void **secLayerHandle);

NMASAPI void NMAS_AuthNCredsFree(
    nuint32 authNCredsCnt,
    NMAS_AuthNCred **authNCreds);
    
NMASAPI int NMAS_SetIdentity(
    nuint32 nmasHandle,
    nint reserved,
    nint ndsContext);

NMASAPI int NMAS_SetSASLIdentity(
    nuint32 nmasHandle,
    nint reserved);

NMASAPI int NMAS_GetUserCredential(
    nuint32 nmasHandle,
    nuint32 *publicKeySize,
    char *publicKey,
    nuint32 *credentialSize,
    char *credential,
    nuint32 *signatureSize,
    char *signature);

// Obsolete
NMASAPI int NMAS_Logout(
    nuint32 nmasHandle,
    nint reserved,
    nint ndsContext);

NMASAPI int NMAS_PutLoginConfig(
    int reserved,
    unicode *objectDN,
    nuint32 methodIDLen,
    nuint32 *methodID,
    unicode *vendorTag,
    nuint32 dataLen,
    void *data);

NMASAPI int NMAS_GetLoginConfig(
    int reserved,
    unicode *objectDN,
    nuint32 methodIDLen,
    nuint32 *methodID,
    unicode *vendorTag,
    nuint32 *dataLen,
    void *data);
                                        
NMASAPI int NMAS_DeleteLoginConfig(
    int reserved,
    unicode *objectDN,
    nuint32 methodIDLen,
    nuint32 *methodID,
    unicode *vendorTag);

NMASAPI int NMAS_PutLoginSecret(
    int reserved,
    unicode *objectDN,
    nuint32 methodIDLen,
    nuint32 *methodID,
    unicode *vendorTag,
    nuint32 dataLen,
    void *data);

NMASAPI int NMAS_DeleteLoginSecret(
    int reserved,
    unicode *objectDN,
    nuint32 methodIDLen,
    nuint32 *methodID,
    unicode *vendorTag);

NMASAPI int NMAS_PolicyRefresh(
    int reserved);
    
NMASAPI int NMAS_ConsoleCommand(
    int reserved,
    char *commandLine);

NMASAPI int NMAS_CheckLoginPolicy(
    unicode *subjDN,
    unicode *objectDN,
    nuint32 flags,
    nuint32 addressSize,
    nuint8 *address);

NMASAPI int NMAS_SetAddressPolicy(
    unicode *subjDN,
    unicode *objectDN,
    nuint32 flags,
    nuint32 addressSize,
    nuint8 *address);

NMASAPI int NMAS_GenerateRandPwdWithPolicy(
    int flags,
    char *xmlPolicy,
    nuint32 *pwdLen, // number of unicode characters including null
    unicode *pwd);

#endif
     
